#include <Adafruit_NeoPixel.h>
#include <ThreeWire.h>  
#include <RtcDS1302.h>

#define PIN_LED 0
#define PIN_RST 2
#define PIN_DAT 3
#define PIN_CLK 4
#define LED_COUNT 17
#define PURE_BINARY

long secondsOld;

// Create the pixels and RTC objects.
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(LED_COUNT, PIN_LED, NEO_GRB + NEO_KHZ800);
ThreeWire myWire(PIN_DAT, PIN_CLK, PIN_RST);
RtcDS1302<ThreeWire> Rtc(myWire);


// *** Initialize the clock.
void setup(){
  secondsOld = 0;
  pixels.begin();
  Rtc.Begin();
  // The next line will "capture" the date and time when the sketch was compiled.
  RtcDateTime compiled = RtcDateTime(__DATE__, __TIME__);
  // If the time of the real-time clock is "smaller" than this, it is likely that its battery is empty and it lost track of the actual time.
  // Hence, the next lines will make the RTC chip running, and if it reports a "smaller" time, the code will update the RTC with the compile time.
  if(Rtc.GetIsWriteProtected()){
    Rtc.SetIsWriteProtected(false);
  }
  if(!Rtc.IsDateTimeValid()){
    Rtc.SetDateTime(compiled);
  }
  if(!Rtc.GetIsRunning()){
    Rtc.SetIsRunning(true);
  }
  RtcDateTime now = Rtc.GetDateTime();
  if(now < compiled){
    Rtc.SetDateTime(compiled);
  }
}

// *** Infinite loop: Check whether the time has changed and modify the LEDs accordingly.
void loop(){
  long hours, minutes, seconds;
  int i;
  bool hasUpdate = false;

  // ****************** Read the time from the RTC chip and "disassemble" it.
  RtcDateTime now = Rtc.GetDateTime();
  if(now.IsValid()){
    hours = now.Hour();
    minutes = now.Minute();
    seconds = now.Second();
    if(seconds != secondsOld){
      secondsOld = seconds;
      hasUpdate = true;
    }
  }
  else{
    // Failed to read the time --> Set all LEDs to RED to show the error.
    for(i = 0; i < 17; i++){
      pixels.setPixelColor(i, 0x0000FF); 
    }
  }
  if(!hasUpdate){
    return;
  }
  // ****************** Let's switch the LEDs!
  // Hour LEDs.
  for(i = 0; i < 5; i++){
    pixels.setPixelColor(i, (0x0000FF + i * 0x101000) * ((hours >> (4 - i)) & 0x01)); 
  }
#ifdef PURE_BINARY // Display minutes and secons in real binary coding.
  // Minutes LEDs.
  for(i = 0; i < 6; i++){
    pixels.setPixelColor(i+5, (0x00FF00 + i * 0x010010) * ((minutes >> (5 - i)) & 0x01)); 
  }
  // Seconds LEDs.
  for(i = 0; i < 6; i++){
    pixels.setPixelColor(i+11, (0xFF0000 + i * 0x000404) * ((seconds >> (5 - i)) & 0x01)); 
  }
#else // Display minutes and hours using the first LED as "30" the second LED as "15" and the remaining 4 as "0...14".
  // Minutes LEDs.
  // Set 30 minutes LED.
  pixels.setPixelColor(5, 0x00FF00*(long)(minutes >= 30));
  minutes %= 30;
  // Set 15 minutes LED.
  pixels.setPixelColor(6, 0x10FF10*(long)(minutes >= 15));
  minutes %= 15;
  // Set 0...14 minutes LEDs.
  for(i = 2; i < 6; i++){
    pixels.setPixelColor(i+5, (0x00FF00 + i * 0x100010) * ((minutes >> (5 - i)) & 0x01)); 
  }
  // Seconds LEDs.
  // Set 30 seconds LED.
  pixels.setPixelColor(11, 0xFF0000*(long)(seconds >= 30));
  seconds %= 30;
  // Set 15 minutes LED.
  pixels.setPixelColor(12, 0xFF1010*(long)(seconds >= 15));
  seconds %= 15;
  // Set 0...14 seconds LEDs.
  for(i = 2; i < 6; i++){
    pixels.setPixelColor(i+11, (0xFF0000 + i * 0x001010) * ((seconds >> (5 - i)) & 0x01)); 
  }
#endif
  pixels.show(); // This sends the updated pixel color to the hardware.
}